# Copyright 2019 QuantRocket LLC - All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from trading_calendars import register_calendar_type, register_calendar_alias
from .exchange_calendar_aeb import AEBExchangeCalendar
from .exchange_calendar_asx import ASXExchangeCalendar
from .exchange_calendar_bm import BMExchangeCalendar
from .exchange_calendar_bux import BUXExchangeCalendar
from .exchange_calendar_bvl import BVLExchangeCalendar
from .exchange_calendar_bvme import BVMEExchangeCalendar
from .exchange_calendar_cboe import CBOEExchangeCalendar
from .exchange_calendar_ebs import EBSExchangeCalendar
from .exchange_calendar_enextbe import ENEXTBEExchangeCalendar
from .exchange_calendar_fwb import FWBExchangeCalendar
from .exchange_calendar_globex import GlobexExchangeCalendar
from .exchange_calendar_iceus import ICEUSExchangeCalendar
from .exchange_calendar_kse import KSEExchangeCalendar
from .exchange_calendar_lse import LSEExchangeCalendar
from .exchange_calendar_mexi import MEXIExchangeCalendar
from .exchange_calendar_moex import MOEXExchangeCalendar
from .exchange_calendar_nyse import NYSEExchangeCalendar
from .exchange_calendar_ose import OSEExchangeCalendar
from .exchange_calendar_sbf import SBFExchangeCalendar
from .exchange_calendar_sehk import SEHKExchangeCalendar
from .exchange_calendar_sehkntl import SEHKNTLExchangeCalendar
from .exchange_calendar_sfb import SFBExchangeCalendar
from .exchange_calendar_sgx import SGXExchangeCalendar
from .exchange_calendar_tse import TSEExchangeCalendar
from .exchange_calendar_tsej import TSEJExchangeCalendar
from .exchange_calendar_vse import VSEExchangeCalendar
from .exchange_calendar_wse import WSEExchangeCalendar

_quantrocket_ibkr_calendar_factories = {
    "AEB": AEBExchangeCalendar,
    "ASX": ASXExchangeCalendar,
    "BM": BMExchangeCalendar,
    "BUX": BUXExchangeCalendar,
    "BVL": BVLExchangeCalendar,
    "BVME": BVMEExchangeCalendar,
    "CBOE": CBOEExchangeCalendar,
    "EBS": EBSExchangeCalendar,
    "ENEXT.BE": ENEXTBEExchangeCalendar,
    "FWB": FWBExchangeCalendar,
    "GLOBEX": GlobexExchangeCalendar,
    "ICEUS": ICEUSExchangeCalendar,
    "KSE": KSEExchangeCalendar,
    "LSE": LSEExchangeCalendar,
    "MEXI": MEXIExchangeCalendar,
    "MOEX": MOEXExchangeCalendar,
    "NYSE": NYSEExchangeCalendar,
    "OSE": OSEExchangeCalendar,
    "SEHK": SEHKExchangeCalendar,
    "SEHKNTL": SEHKNTLExchangeCalendar,
    "SBF": SBFExchangeCalendar,
    "SFB": SFBExchangeCalendar,
    "SGX": SGXExchangeCalendar,
    "TSE": TSEExchangeCalendar,
    "TSEJ": TSEJExchangeCalendar,
    "VSE": VSEExchangeCalendar,
    "WSE": WSEExchangeCalendar,
}

_quantrocket_ibkr_calendar_aliases = {
    "NASDAQ": "NYSE",
    "ARCA": "NYSE",
    "AMEX": "NYSE",
    "BATS": "NYSE",
    "IEX": "NYSE",
    "PINK": "NYSE",
    "ENEXT": "ENEXT.BE",
    "SEHKSZSE": "SEHKNTL",
}

_quantrocket_mic_aliases = {

    # Other US Mics
    "XNAS": "XNYS",
    "ARCX": "XNYS",
    "XASE": "XNYS",
    "BATS": "XNYS",
    "PINX": "XNYS",
    "PSGM": "XNYS",
    "OTCB": "XNYS",
    "OTCQ": "XNYS",
}

quantrocket_calendar_factories = {}

for name, calendar in _quantrocket_ibkr_calendar_factories.items():
    register_calendar_type(name, calendar, force=True)
    quantrocket_calendar_factories[name] = calendar


for alias, real_name in _quantrocket_ibkr_calendar_aliases.items():
    register_calendar_alias(alias, real_name, force=True)
    quantrocket_calendar_factories[alias] = quantrocket_calendar_factories[real_name]

for alias, real_name in _quantrocket_mic_aliases.items():
    register_calendar_alias(alias, real_name, force=True)

quantrocket_calendar_names = sorted(quantrocket_calendar_factories.keys())



#!/usr/bin/env python

# Copyright 2019 QuantRocket LLC - All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import pandas as pd
import json
from quantrocket_trading_calendars.calendar_utils import quantrocket_calendar_factories
from trading_calendars.calendar_utils import _default_calendar_factories

def get_exchange_status(exchange, dt):
    """
    Returns the exchange status at the specified datetime.
    """
    try:
        calendar_cls = quantrocket_calendar_factories[exchange]
    except KeyError:
        calendar_cls = _default_calendar_factories[exchange]

    asof_datetime = pd.Timestamp(dt, tz=calendar_cls.tz)
    start = asof_datetime - pd.Timedelta(days=30)
    start = pd.Timestamp(start.date(), tz="UTC")
    end = asof_datetime + pd.Timedelta(days=30)
    end = pd.Timestamp(end.date(), tz="UTC")
    calendar = calendar_cls(start=start, end=end)

    is_open = calendar.is_open_on_minute(asof_datetime)
    # Note: The `trading_calendars` package sets exchange open times 1 minute
    # later than the actual open. For example, the exchange hours for NYSE
    # are 9:31-16:00 in `trading_calendars`, even though NYSE actually opens
    # at 9:30. This behavior reflects the needs of zipline. To deal with
    # this, we consider the exchange open if it is open this minute, or next
    # minute.
    if not is_open:
        is_open = calendar.is_open_on_minute(asof_datetime + pd.Timedelta(minutes=1))

    if is_open:
        # Rewind open 1 minute
        since = calendar.previous_open(asof_datetime) - pd.Timedelta(minutes=1)
        until = calendar.next_close(asof_datetime)
    else:
        since = calendar.previous_close(asof_datetime)
        # Rewind open 1 minute
        until = calendar.next_open(asof_datetime) - pd.Timedelta(minutes=1)

    since = since.tz_convert(asof_datetime.tz.zone).strftime("%Y-%m-%dT%H:%M:%S")
    until = until.tz_convert(asof_datetime.tz.zone).strftime("%Y-%m-%dT%H:%M:%S")

    return dict(
        status="open" if is_open else "closed",
        since=since,
        until=until)

def main():

    parser = argparse.ArgumentParser(
        description="check the status of an exchange at the specified time")
    parser.add_argument(
        "exchange",
        help="the IB exchange code")
    parser.add_argument(
        "dt",
        help="the ISO format datetime to check")

    args = parser.parse_args()
    args = vars(args)

    status = get_exchange_status(args["exchange"], args["dt"])
    print(json.dumps(status))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from setuptools import setup
import versioneer

setup(
    name='quantrocket-trading-calendars',
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description='Trading calendars for QuantRocket-supported exchanges',
    maintainer='QuantRocket LLC',
    maintainer_email='support@quantrocket.com',
    url='https://github.com/quantrocket-llc/ib-trading-calendars',
    license='Apache 2.0',
    packages=['quantrocket_trading_calendars'],
    entry_points = {
        'console_scripts': ['exchange-status=quantrocket_trading_calendars.status:main'],
    },
    install_requires=[
        'trading_calendars>=1.11.0'
    ]
)
